/**
 * Utility functions to support the JSDoc plugin framework.
 * @module jsdoc/plugins
 */
const dictionary = require('jsdoc/tag/dictionary');

function addHandlers(handlers, parser) {
    Object.keys(handlers).forEach(eventName => {
        parser.on(eventName, handlers[eventName]);
    });
}

exports.installPlugins = (plugins, parser) => {
    function initPlugin(pluginModule, pluginOptions) {
        const rawPlugin = require(pluginModule);
        let plugin = rawPlugin
        
        if (typeof rawPlugin === 'function') {
            plugin = rawPlugin(pluginOptions)
        }

        // allow user-defined plugins to...
        // ...register event handlers
        if (plugin.handlers) {
            addHandlers(plugin.handlers, parser);
        }

        // ...define tags
        if (plugin.defineTags) {
            plugin.defineTags(dictionary);
        }

        // ...add a Mozilla Parser API node visitor
        if (plugin.astNodeVisitor) {
            parser.addAstNodeVisitor(plugin.astNodeVisitor);
        }
    }

    for (let pluginModule of plugins) {
        if (Array.isArray(pluginModule)) {
            const [plugin, pluginOptions] = pluginModule
            initPlugin(plugin, pluginOptions)
        } else {
            initPlugin(pluginModule, null)
        }
    }
};
